@@ -20,16 +20,23 @@ from . import util
 
 # magic
 def get_vector(token_span_doc):
+    # yanked v0.4.2: unwanted getting vectors from underlying model
+    # # Use the vocab if the user has set there something.
+    # # The comparison is done on the string
+    # if token_span_doc.vocab.has_vector(token_span_doc.text):
+    #     return token_span_doc.vocab.get_vector(token_span_doc.text)
     doc = token_span_doc.doc
     use_model_url = doc._.use_model_url
+    preprocessor_url = doc._.preprocessor_url
     # if not use_model_url:
-    model = UniversalSentenceEncoder.get_model(use_model_url)
+    model = UniversalSentenceEncoder.get_model(use_model_url, preprocessor_url)
     vector = model.embed_one(token_span_doc)
     return vector
 
 
 # install/register the extensions
 Doc.set_extension('use_model_url', default=None, force=True)
+Doc.set_extension('preprocessor_url', default=None, force=True)
 Token.set_extension('universal_sentence_encoding', getter=get_vector, force=True)
 Span.set_extension('universal_sentence_encoding', getter=get_vector, force=True)
 Doc.set_extension('universal_sentence_encoding', getter=get_vector, force=True)
@@ -37,11 +44,13 @@ Doc.set_extension('universal_sentence_encoding', getter=get_vector, force=True)
 # the pipeline stage factory
 @Language.factory('universal_sentence_encoder', default_config={
     'use_model_url': None,
+    'preprocessor_url': None,
     'model_name': None,
     'enable_cache': True,
     'debug': False
 })
-def use_model_factory(nlp, name, use_model_url, model_name, enable_cache, debug):
+def use_model_factory(nlp, name, use_model_url, preprocessor_url, model_name, enable_cache, debug):
+    preprocessor_url_config = None
     if debug:
         print('use_model_factory:', nlp, 'use_model_url', use_model_url, 'model_name', model_name)
     if use_model_url:
@@ -54,6 +63,7 @@ def use_model_factory(nlp, name, use_model_url, model_name, enable_cache, debug)
             raise ValueError(f'Parameter {model_name} must be one of {util.configs.keys()}')
         config = util.configs[model_name]
         model_url = config['use_model_url']
+        preprocessor_url_config = config.get('preprocessor_url', None)
     else:
         # inherit from nlp object
         # the language code needs to match
@@ -76,9 +86,14 @@ def use_model_factory(nlp, name, use_model_url, model_name, enable_cache, debug)
                 model_name = model_name_best
         config = util.configs[model_name]
         model_url = config['use_model_url']
+        preprocessor_url_config = config.get('preprocessor_url', None)
 
+
+    if not preprocessor_url and preprocessor_url_config:
+        preprocessor_url = preprocessor_url_config
+    
     if debug:
-        print('model_url=', model_url)
+        print('model_url=', model_url, 'preprocessor_url=', preprocessor_url)
 
     if config and config['spacy_base_model'] == 'xx':
         # double check `multi` extra is installed
@@ -86,7 +101,7 @@ def use_model_factory(nlp, name, use_model_url, model_name, enable_cache, debug)
             raise ValueError('This multilanguage model requires tensorflow_text. Install it with: pip install spacy-universal-sentence-encoder[multi]')
 
     
-    model = UniversalSentenceEncoder(model_url, enable_cache, debug)
+    model = UniversalSentenceEncoder(model_url, preprocessor_url, enable_cache, debug)
     return model
 
 class UniversalSentenceEncoder(object):
@@ -95,23 +110,25 @@ class UniversalSentenceEncoder(object):
 
     working_pid = None
 
-    def __init__(self, model_url, enable_cache=True, debug=False):
+    def __init__(self, model_url, preprocessor_url, enable_cache=True, debug=False):
         
         self.model_url = model_url
+        self.preprocessor_url = preprocessor_url
         self.enable_cache=enable_cache
         self.debug=debug
         # load it now so that when the extension getter will call it, the model will be already loaded
-        _ = UniversalSentenceEncoder.get_model(self.model_url, self.enable_cache, self.debug)
+        _ = UniversalSentenceEncoder.get_model(self.model_url, preprocessor_url, self.enable_cache, self.debug)
 
 
     def __call__(self, doc):
         doc._.use_model_url = self.model_url
+        doc._.preprocessor_url = self.preprocessor_url
         set_hooks(doc)
 
         return doc
 
     @staticmethod
-    def get_model(use_model_url, enable_cache=True, debug=False):
+    def get_model(use_model_url, preprocessor_url, enable_cache=True, debug=False):
         # print('getting', use_model_url)
 
         # PID checking: TensorFlow gets stuck with multiple processes
@@ -130,7 +147,7 @@ class UniversalSentenceEncoder(object):
             model = UniversalSentenceEncoder.models[use_model_url] 
         else:
             # print('model not in cache')
-            model = TFHubWrapper(use_model_url, enable_cache=enable_cache, debug=debug)
+            model = TFHubWrapper(use_model_url, preprocessor_url, enable_cache=enable_cache, debug=debug)
             UniversalSentenceEncoder.models[use_model_url] = model
         return model
 
@@ -148,11 +165,12 @@ def set_hooks(doc):
 def create_nlp(cfg, nlp=None):
     spacy_base_model = cfg['spacy_base_model']
     use_model_url = cfg['use_model_url']
+    preprocessor_url = cfg.get('preprocessor_url', None)
     
     if not nlp:
         nlp = spacy.blank(spacy_base_model)
         nlp.add_pipe('sentencizer')
-    nlp.add_pipe('universal_sentence_encoder', config={'use_model_url': use_model_url})
+    nlp.add_pipe('universal_sentence_encoder', config={'use_model_url': use_model_url, 'preprocessor_url': preprocessor_url})
     return nlp
 
 
@@ -163,7 +181,7 @@ class TFHubWrapper(object):
     model: Any
 
 
-    def __init__(self, use_model_url, enable_cache=True, debug=False):
+    def __init__(self, use_model_url, preprocessor_url, enable_cache=True, debug=False):
         self.embed_cache = {}
 
         logging.set_verbosity(logging.ERROR)
@@ -180,11 +198,19 @@ class TFHubWrapper(object):
         if debug:
             print(f'module {self.model_url} loaded')
 
+        # cmlm requires preprocessor before
+        if preprocessor_url:
+            preprocessor = hub.load(preprocessor_url)
+            self.model.preprocessor = preprocessor
+
 
     def embed(self, texts: List[str]):
         """Embed multiple texts"""
         # print('embed TFHubWrapper called')
-        result = self.model(texts)
+        if hasattr(self.model, 'preprocessor'):
+            result = self.model(self.model.preprocessor(texts))['default']
+        else:
+            result = self.model(texts)
         result = np.array(result)
         return result
 
